from six import string_types
from sys import exit
from time import time
from types import FunctionType

from nozdormu.loader import BenchLoader
from nozdormu.runner import BenchRunner

class BenchProgram:
    """This program loads and runs the tests"""

    module = None # For testing purposes

    def __init__(self, module='__main__', benchLoader=BenchLoader,
                 benchRunner=BenchRunner):
        self.benchLoader = benchLoader
        self.benchRunner = benchRunner
        if isinstance(module, string_types):
            self.module = __import__(module)
            for part in module.split('.')[1:]:
                self.module = getattr(self.module, part)
        self.load()
        self.run()

    def load(self):
        loader = self.benchLoader()
        self.bench = loader.loadFromModule(self.module)

    def run(self):
        runner = self.benchRunner()
        self.result = runner.run(self.bench)

main = BenchProgram




from __future__ import print_function

import gc
import json
import sys

if sys.platform == 'win32':
    from time import clock as time
else:
    from time import time

from nozdormu.util import format_time

termc = {
    'def': '\033[0m',
    'red': '\033[91m',
    'green': '\033[92m',
    'yellow': '\033[93m',
    'cyan': '\033[96m',
}

class BenchRunner:
    """The standard runner for a BenchSuite"""

    def run(self, suite):
        """
        Run the benchmarks interleaved until all tests are finished,
        runs them grouped by batch. Compares to a possibly existing
        json baseline (.nozdormu) and writes a new one.  Benchmarks are
        run until they accumulate 1ms of runtime, but at least 16 times.
        """
        totalStart = time()
        noBatches = 0
        noBenchs = 0
        self.results = []

        try: # to open the baseline file
            with open('.nozdormu', 'r') as f:
                baseline = json.loads(f.read())
        except IOError:
            baseline = None

        self.output('Starting benchmark session\n')
        for batch in suite:
            gcold = gc.isenabled()
            gc.disable()

            baselineBatch = None
            if baseline:
                for b in baseline:
                    if b['batch'] == batch.__repr__():
                        baselineBatch = b['results']
                        break

            batchResults = []
            noBatches += 1
            benchsRunning = [b for b in batch]
            noBenchs += len(benchsRunning)
            batchStart = time()
            self.output('  Running Batch: {}'.format(batch))
            while len(benchsRunning) >= 1:
                for bench in benchsRunning:
                    if bench.totalTime < 0.001 or bench.count < 16:
                        bench.run()
                    else:
                        benchsRunning.remove(bench)

                        baselineBench = None
                        if baselineBatch:
                            for b in baselineBatch:
                                if b['name'] == bench.methodName:
                                    baselineBench = b
                                    btime = b['time']
                                    break
                        bexact = bench.exact()
                        if not baselineBench:
                            baseComp = '{}{}{}'.format(termc['yellow'], 'new',
                                       termc['cyan'])
                        elif btime >= bexact:
                            baseComp = '{}-{} / {:.1f}%{}'.format(
                                termc['green'], format_time(btime - bexact),
                                100 - bexact * 100 / btime,
                                termc['cyan'])
                        else:
                            baseComp = '{}+{} / {:.1f}%{}'.format(
                                termc['red'], format_time(bexact - btime),
                                100 - btime * 100 / bexact,
                                termc['cyan'])

                        self.output('    {}{}: {} ({}){}'.format(termc['cyan'],
                              bench, bench.results(), baseComp, termc['def']))
                        batchResults.append({'name': bench.methodName,
                                             'time': bexact,
                                             'runs': bench.count, })

            self.output('  Batch finished, time: {}\n'.format(
                        format_time(time() - batchStart)))
            self.results.append({'batch': batch.__repr__(),
                                 'results': batchResults,})

            if gcold:
                gc.enable()

        self.output('Benchmarking finished\n'
                    '{} batches, {} benchmarks\n'
                    'total time: {}'.format(
                    noBatches, noBenchs, format_time(time() - totalStart)))

        # Write the new baseline
        with open('.nozdormu', 'w') as f:
            f.write(json.dumps(self.results, sort_keys=True, indent=2))

        return self.results

    def output(self, arg):
        """Print wrapper to easily supress output in tests"""
        print(arg)




import sys

if sys.version_info.major == 2:
    from types import ClassType as batchType
else:
    batchType = type

from nozdormu.batch import BenchBatch
from nozdormu.suite import BenchSuite

class BenchLoader:
    """Load a batch of benchmarks and return them"""

    def __init__(self, benchSuite=BenchSuite):
        self.suiteClass = benchSuite

    def loadFromModule(self, module):
        """Get all batches from a module and all benchs from a batch"""
        benchs = []
        for name in dir(module):
            obj = getattr(module, name)
            if isinstance(obj, batchType) and issubclass(obj, BenchBatch):
                benchs.append(self.loadFromBatch(obj))
        return self.suiteClass(benchs)

    def loadFromBatch(self, batch):
        """Get all benchs from a batch and return a loaded suite"""
        isbench = lambda name: name.startswith('bench')
        benchNames = list(filter(isbench, dir(batch)))
        suite = self.suiteClass(map(batch, benchNames))
        suite.batch=batch
        return suite




from setuptools import setup

import nozdormu

def readme():
    try:
        with open('readme.rst') as f:
            return f.read()
    except: # FileNotFound?
        return ''

setup(name='nozdormu',
      version=nozdormu.VERSION,
      description='Python benchmarking for humans and dragons',
      long_description=readme(),
      url='https://github.com/sulami/nozdormu',
      author='Robin Schroer',
      author_email='sulami@peerwire.org',
      license='MIT',
      packages=['nozdormu'],
      install_requires=[
          'six>=1.9.0',
      ],
      zip_safe=False,
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: MIT License',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Environment :: Console',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Topic :: Software Development',
          'Topic :: Software Development :: Testing',
      ],
      )

